'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _immutableBuilder = require('immutable-builder');

var _immutableBuilder2 = _interopRequireDefault(_immutableBuilder);

var _qs = require('qs');

var _qs2 = _interopRequireDefault(_qs);

var _QueryStringParser = require('./QueryStringParser');

var _QueryStringParser2 = _interopRequireDefault(_QueryStringParser);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = Resource;


function Resource(_type, _baseUrl) {
  var _idKey = arguments.length <= 2 || arguments[2] === undefined ? 'id' : arguments[2];

  return _immutableBuilder2.default.new({
    _attributes: null,
    _baseUrl: _baseUrl,
    _elements: [],
    _links: {},
    _idKey: _idKey,
    _includes: [],
    _meta: {},
    // relationship: {name: '', sourceKey: '', destKey: 'id', resource: ''}
    _relationships: [],
    _type: _type,
    _querystring: null,
    _paging: null,
    _relatedResources: {},

    _urlGenerator: function _urlGenerator(type, id) {
      var url = this._baseUrl + '/' + type;
      if (id) url += '/' + id;
      return url;
    },
    querystring: function querystring(value) {
      if (arguments.length) {
        if (!(value instanceof _QueryStringParser2.default)) {
          throw new Error('querystring must be instance of QueryStringParser');
        } else {
          return this.new(function (_this) {
            return _this._querystring = value;
          });
        }
      } else {
        return this._querystring;
      }
    },
    paging: function paging(count, exists) {
      if (arguments.length) {
        return this.new(function (_this) {
          return _this._paging = { count: count, exists: exists };
        });
      } else {
        return this._paging;
      }
    },
    relatedResources: function relatedResources() {
      return this.new(function () {
        var _this2 = this;

        var relationships = this.relationships();

        if (relationships.length) {
          var attributes = this.attributes();
          var elements = this.elements();

          if (attributes) {
            this.processRelationships(relationships, { attributes: attributes, links: {} });
          }

          if (elements) {
            _lodash2.default.forEach(elements, function (attributes) {
              return _this2.processRelationships(relationships, { attributes: attributes, links: {} });
            });
          }
        }
      })._relatedResources;
    },
    toJSON: function toJSON() {
      var _this = this.new();
      var links = _this.links();
      var relationships = _this.relationships();
      var attributes = _this.attributes();
      var meta = _this.meta();
      var elements = _this.elements();
      var includes = _this.includes();
      var obj = void 0;

      links.$self = function (resource) {
        return _this._urlGenerator(_this._type, resource[_this._idKey]);
      };
      var linksTemplate = compileLinks(links);

      if (attributes) {
        obj = _this.processAttributes(linksTemplate, attributes);

        if (relationships.length) {
          _this.processRelationships(relationships, obj);
        }
      } else {
        obj = {
          links: {
            $self: _this.pagingLink()
          }
        };
      }

      if (_lodash2.default.keys(meta).length) {
        obj.meta = meta;
      }

      if (elements.length) {
        obj.elements = _this.elements().map(_this.processAttributes.bind(_this, linksTemplate)).map(_this.processRelationships.bind(_this, relationships));
      }

      if (includes.length) {
        obj.includes = includes.map(function (x) {
          return x.toJSON ? x.toJSON() : x;
        });
      }

      _this.createPagingLinks(obj);
      return obj;
    },
    createPagingLinks: function createPagingLinks(obj) {
      var options = this.paging();
      var page = this._querystring && this._querystring.page();

      if (options && page) {
        var count = options.count;
        var exists = options.exists;

        var pageCount = Math.ceil(count / page.size);
        var l = obj.links;

        if (!obj.meta) obj.meta = {};

        obj.meta.count = count;
        obj.meta.pageCount = pageCount;
        obj.meta.size = page.size;

        switch (page.method) {
          case 'number':
            l.$first = this.pagingLink({ number: 1 });
            l.$last = this.pagingLink({ number: pageCount });

            if (page.number > 1) {
              l.$previous = this.pagingLink({ number: page.number - 1 });
            }

            if (page.number < pageCount) {
              l.$next = this.pagingLink({ number: page.number + 1 });
            }

            obj.meta.number = page.number;
            break;

          case 'offset':
            l.$first = this.pagingLink({ offset: 0 });
            l.$last = this.pagingLink({ offset: (pageCount - 1) * page.size });

            if (page.offset > 0) {
              l.$previous = this.pagingLink({
                offset: Math.max(0, page.offset - page.size)
              });
            }

            if (page.offset < count - page.size) {
              l.$next = this.pagingLink({ offset: page.offset + page.size });
            }

            obj.meta.offset = page.offset;
            break;

          case 'after':
            var prev = void 0,
                next = void 0;

            if (page.reverse) {
              prev = exists;
              next = typeof page.after !== 'undefined';
            } else {
              prev = typeof page.after !== 'undefined';
              next = exists;
            }

            l.$first = this.pagingLink({ after: '' });
            l.$last = this.pagingLink({ before: '' });

            if (prev) {
              l.$previous = this.pagingLink({
                before: (0, _stringify2.default)(this._elements[0][page.field])
              });
            }

            if (next) {
              l.$next = this.pagingLink({
                after: (0, _stringify2.default)(this._elements[this._elements.length - 1][page.field])
              });
            }

            obj.meta.after = page.after;
            break;
        }
      }
    },
    pagingLink: function pagingLink(page, id) {
      var querystring = { page: page };

      if (this._querystring) _lodash2.default.defaults(querystring, this._querystring.qs);

      if (page) querystring.page.size = this._querystring.page().size;

      var str = _qs2.default.stringify(querystring);
      if (str) str = '?' + str;

      return this._urlGenerator(this._type, id) + str;
    },
    processAttributes: function processAttributes(linksTemplate, attributes) {
      return {
        links: linksTemplate(attributes),
        attributes: attributes
      };
    },
    processRelationships: function processRelationships(relationships, obj) {
      var attributes = obj.attributes;
      var keys = _lodash2.default.keys(attributes);

      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = (0, _getIterator3.default)(relationships), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var relationship = _step.value;

          if (_lodash2.default.includes(keys, relationship.sourceKey)) {
            var url = void 0;
            var operation = void 0;

            if (relationship.destKey === this._idKey || !relationship.destKey) {
              url = this._urlGenerator(relationship.resource, attributes[relationship.sourceKey]);
              operation = 'get';
            } else {
              var filter = {};
              filter[relationship.destKey] = attributes[relationship.sourceKey];

              url = this._urlGenerator(relationship.resource) + (0, _QueryStringParser.querystringify)({ filter: filter });
              operation = 'list';
            }

            obj.links[relationship.name] = url;

            if (typeof relationship.discard === 'undefined') {
              relationship.discard = relationship.destKey === this._idKey || !relationship.destKey;
            }

            this._relatedResources[url] = {
              resource: relationship.resource,
              name: relationship.name,
              key: attributes[relationship.sourceKey],
              operation: operation
            };

            if (relationship.discard) {
              delete attributes[relationship.sourceKey];
            }
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return obj;
    }
  });
};

function compileLinks(links) {
  if (links && !_lodash2.default.isFunction(links)) {
    var _ret = function () {
      var template = _lodash2.default.mapValues(links, function (x) {
        return _lodash2.default.isFunction(x) ? x : _lodash2.default.template(x);
      });
      return {
        v: function v(x) {
          return _lodash2.default.mapValues(template, function (fn) {
            return fn(x);
          });
        }
      };
    }();

    if ((typeof _ret === 'undefined' ? 'undefined' : (0, _typeof3.default)(_ret)) === "object") return _ret.v;
  }
}
